#
# Argument initialization
#

$nextarg = "none"
$DebugPort = "unassigned"
$targetcomputer = "."
$VMName = ""
$VMGuid = ""
$AutoAssign = "false"
$DebugOff = "false"

function funHelp()
{
$helpText=@"

DESCRIPTION:
NAME: kdnetdebugvm.ps1
Displays (and optionally sets) the port used to network debug a VM.

PARAMETERS: 
-computerName Specifies the name of the computer on which to run the script
-help         Displays help
-vmname       (optional) Name of the VM of interest
-vmguid       (optional) GUID of the VM of interest
-port         (optional) Network port to use for debugging
-debugoff
-autoassign

Either vmname or vmguid must be specified to identify the VM, but not both.
Note that vmname may not uniquely identify the VM, but vmguid does.

SYNTAX:
kdnetdebugvm.ps1 [-computerName targetcomputer] [-vmname NameOfVM] [-vmguid GuidOfVM] [-port PortNumber]

"@
$helpText
exit
}


foreach ($argument in $args)
{
    # parse commands with no following arguments
    switch ($argument)
    {
        "?"     {funHelp}
        "help"  {funHelp}
        "-help" {funHelp}
        "/?"    {funHelp}
        "-?"    {funHelp}
        "autoassign"    {$AutoAssign = "true"}
        "-autoassign"   {$AutoAssign = "true"}
        "/autoassign"   {$AutoAssign = "true"}
        "debugoff"        {$DebugOff = "true"}
        "-debugoff"       {$DebugOff = "true"}
        "/debugoff"       {$DebugOff = "true"}
        default {}
    }

    # parse values that followed a switch

    switch ($nextarg)
    {
        "vmname"        {$VMName = $argument}
        "-vmname"       {$VMName = $argument}
        "/vmname"       {$VMName = $argument}
        "vmguid"        {$VMGuid = $argument}
        "-vmguid"       {$VMGuid = $argument}
        "/vmguid"       {$VMGuid = $argument}
        "port"          {$DebugPort = $argument}
        "-port"         {$DebugPort = $argument}
        "/port"         {$DebugPort = $argument}
        "computername"  {$targetcomputer = $argument}
        "-computername" {$targetcomputer = $argument}
        "/computername" {$targetcomputer = $argument}
        default         {}
    }

    $nextarg = $argument
}

if (($VMName -eq "") -and ($VMGuid -eq ""))
{
    funHelp
}

if (($VMName -ne "") -and ($VMGuid -ne ""))
{
    funHelp
}

$ns = "root\virtualization\v2"
$VMWPName = "$env:windir\system32\vmwp.exe"

#Get a VMManagementService object
$VMManagementService = gwmi -class "Msvm_VirtualSystemManagementService" -namespace $ns -computername $targetcomputer

#Get the VM object that we want to modify
if ($VMName -ne "")
{
    $VM = Get-VM -computername $targetcomputer -VMName $VMName
}

if ($VMGuid -ne "")
{
    $VM = Get-VM -computername $targetcomputer -Id $VMGuid
}

#Get the VirtualSystemGlobalSettingData of the VM we want to modify
$VMSystemGlobalSettingData = gwmi -namespace $ns -computername $targetcomputer -class Msvm_VirtualSystemSettingData | ? { $_.ConfigurationID -eq $VM.Id }

# Set a new debugport
if ($DebugPort -ne "unassigned")
{
    #Change the ElementName property
    $VMSystemGlobalSettingData.DebugPort = $DebugPort
    $VMSystemGlobalSettingData.DebugPortEnabled = 1

    $VMManagementService.ModifySystemSettings($VMSystemGlobalSettingData.GetText(1))
    $FWRuleName = "SynthDebugInboundRule-$DebugPort"
    New-NetFirewallRule -DisplayName $FWRuleName -Program $VMWPName -Protocol UDP -Action Allow -Direction Inbound -LocalPort $DebugPort
}

# Enable auto assigned debug ports
if ($AutoAssign -ne "false")
{
    #Change the ElementName property
    $VMSystemGlobalSettingData.DebugPortEnabled = 2
    $VMManagementService.ModifySystemSettings($VMSystemGlobalSettingData.GetText(1))
    Write-Host -Foreground Yellow "Firewall Ports for autoassign mode can be opened only after the VM is started."
}

# Turn off debugging
if ($DebugOff -ne "false")
{
    $DebugPort = $VMSystemGlobalSettingData.DebugPort
    #Change the ElementName property
    $VMSystemGlobalSettingData.DebugPortEnabled = 0
    $VMSystemGlobalSettingData.DebugPort = 0
    $VMManagementService.ModifySystemSettings($VMSystemGlobalSettingData.GetText(1))
    # May throw an exception if the rule did not exist already.
    # If two rules exist with the same name, both will be deleted.
    if ($DebugPort -ne 0)
    {
        $FWRuleName = "SynthDebugInboundRule-$DebugPort"
        Remove-NetFirewallRule -DisplayName $FWRuleName
    }
}

$VMSystemGlobalSettingData

exit

# SIG # Begin signature block
# MIIRLwYJKoZIhvcNAQcCoIIRIDCCERwCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDbHDLYuqp9Du8W
# ZSYoLwjEyhkYywkzwemxEtpUqBqajKCCDb0wggYdMIIEBaADAgECAhMzAAAF4Euw
# oimM649KAAAAAAXgMA0GCSqGSIb3DQEBCwUAMH0xCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xJzAlBgNVBAMTHk1pY3Jvc29mdCBEZXZlbG9wbWVu
# dCBQQ0EgMjAxNDAeFw0yMTA4MTIxODA1MThaFw0yMjA5MTUxODA1MThaMHQxCzAJ
# BgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25k
# MR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xHjAcBgNVBAMTFU1pY3Jv
# c29mdCBDb3Jwb3JhdGlvbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEB
# ALFiUv2KdaSmc0HR1y4EPEU3GFLEOEcRoOa4yGBCWf0unlIgbqGduHF747ftuBju
# 66nmUZRSJQJ/JT8bL2NdkLrEEq4GCjrtScGizGBxzuG+HPA4/E/YTm0nDlQV7h4D
# 6octv3m5geXd8gFpQa3Fgyz2FmtGBhQB51wxaaJA3SbaGtHwhmu16gu4jEWEmvXU
# OsIDVau/Xo4G2ym/UYtHvkVNYiYZrhqpRnYmxYhznC35s1pWz0iftzbQZ9bbMi55
# AoOIF0IHchKCsbZyctDFgZH5e1nwH6DQPtB7YUC1zHTPpJytcLZL1BZdqGLaMllw
# cKIoVyuI8Y5cFvfT2ViE+ncCAwEAAaOCAZ0wggGZMDcGA1UdJQQwMC4GCisGAQQB
# gjcKAw0GCisGAQQBgjcKAxsGCisGAQQBgjc9BgEGCCsGAQUFBwMDMB0GA1UdDgQW
# BBSJN6assnMvuc+DAjfKWNoffd7yZzBFBgNVHREEPjA8pDowODEeMBwGA1UECxMV
# TWljcm9zb2Z0IENvcnBvcmF0aW9uMRYwFAYDVQQFEw0yMzExMDUrNDY2MzkxMB8G
# A1UdIwQYMBaAFITdEDZ7C9IIoyZWnyKVJUGFLAnAMF0GA1UdHwRWMFQwUqBQoE6G
# TGh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2lvcHMvY3JsL01pY3Jvc29mdCUy
# MERldmVsb3BtZW50JTIwUENBJTIwMjAxNC5jcmwwagYIKwYBBQUHAQEEXjBcMFoG
# CCsGAQUFBzAChk5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpb3BzL2NlcnRz
# L01pY3Jvc29mdCUyMERldmVsb3BtZW50JTIwUENBJTIwMjAxNC5jcnQwDAYDVR0T
# AQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEAXri933VYhjKDpmLJ9DtotK0uLbH5
# KnniIE64RLlb5sAXpYAecN4fWqu6s4Gzr73CMif4ShujuPgRti5BcujIWoynZ/Uo
# HD396BnLwJ1lsuJZXd7RDnb0qL7sD///2YVrfC3QHmStwTstzycbtc1GZsFwBnKB
# BoMai2vHxUhJSEtLcA0YuJZwT6TZ6McbYbOEEfI86AvrEOwtb9wP1C6VnS0epdii
# IGF5T4oc0082AtqZ6mcLJypR2QSer7kPs2wkjqTezHLptvxyq/BDjYz1zrIfvGTQ
# +kxqr7uBCMi+9FBq6OgeBagIyrIFqQE6u7djMwyPJGsQ2fqkwIl45K1pNcusPOgR
# 2XP2EOutQ2EhYFnBzEEiNk74EBFXxHTkgz4m4VNeK8nyIphl6g3QQaN43AcTwrSq
# 7zJ9YNj/uogEEMdsuTHzDxsO0BjDMSdpgWvEvm/dx7VVQkbSzWdqCW1jQbCpRztf
# abvzJZZ9tG49bkPKkUaG5h6ImVOCDTQv771b3FOi1xuxy4kOI2HFLo3XsG5TxpY5
# 61U62WRJeEiMOxhglxcisb8rVHnL4d8liHakZU71/SjSAZpzuqU7LUZ0Pbxtngld
# Sq9eDY/eI8EzVpqSBWr6UljwY+6gLXgOfOjQy34idcNAWvPUK4Wu1azTPoT4XXuo
# 0BXU8frZ93N2gFIwggeYMIIFgKADAgECAhMzAAAAA8b5tMOuvllLAAAAAAADMA0G
# CSqGSIb3DQEBCwUAMIGUMQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3Rv
# bjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0
# aW9uMT4wPAYDVQQDEzVNaWNyb3NvZnQgRGV2ZWxvcG1lbnQgUm9vdCBDZXJ0aWZp
# Y2F0ZSBBdXRob3JpdHkgMjAxNDAeFw0xNDA1MjgxNzMzMzNaFw0yOTA1MjgxNzQz
# MzNaMH0xCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQH
# EwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xJzAlBgNV
# BAMTHk1pY3Jvc29mdCBEZXZlbG9wbWVudCBQQ0EgMjAxNDCCAiIwDQYJKoZIhvcN
# AQEBBQADggIPADCCAgoCggIBAMtTDEH5TUQkyNplTmZsxYKVcHIz87cuhq4WQbzc
# 1Lhp+xLAlL6vlMJ/dpw801TUtYjtj7EMG1FKj+qQ46xO99/SEnuOf3LQNTys6Oln
# zOkyMjHoQ1/fFJpAjxgLmQcTO3GSmx0Z+ZuV9081YThWMrLtZIQY+lqFsZK/AbjG
# lpMEOqjq7ILqzR6CwuYA5Jk9iaXdcSCeiN1eTa1tzwnzqKYTbwwD4QwumLTCowec
# Gm9f03YvvpnulcawlF+xuKZLky8C8cwjwkOWOKW/m3wtbBw1R7iAPOf6nwyc799e
# jM90Cwwzl3/qBNXFM9MPnrCQJdq6/WieybQBbqnvYzMw6QjI65PPW5m/ocpUOsCK
# HZNsitC8L8ci/kDjRHaa0pTfF2VLeyaAztlHM031ZuT8xs1yrH7xcJKDPnIhkMDs
# jf9ZsVuzMlCUxcFse2pWyPkQfEsanmEsoiHuX5ykZOwy0rR7mohCqSxWLheW/pMr
# RAxenyPOAvY7iZjAQfxB1/oofUQB8pGy0bpintgFYwZWpsIYAWb9vUQrTVnC8Jss
# EHQ0byChKj1YzlzwUeEloem/szVxyQ1lxWx1AbfD/LOWk8LYr01ZKv3Mz6bzw7WW
# knqrtEquOKICPeaRkN2j7KklyQjBUbGpIFyMajqR5wkSf87KeM8oQ6p0C1yeEXD/
# xFk1AgMBAAGjggH3MIIB8zAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQUhN0Q
# NnsL0gijJlafIpUlQYUsCcAwZAYDVR0gBF0wWzAGBgRVHSAAMFEGDCsGAQQBgjdM
# g30BATBBMD8GCCsGAQUFBwIBFjNodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
# b3BzL0RvY3MvUmVwb3NpdG9yeS5odG0wGQYJKwYBBAGCNxQCBAweCgBTAHUAYgBD
# AEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB/wQFMAMBAf8wHwYDVR0jBBgwFoAUhXAA
# n3dZHoysPJ93JigZzJrBjzIweAYDVR0fBHEwbzBtoGugaYZnaHR0cDovL3d3dy5t
# aWNyb3NvZnQuY29tL3BraW9wcy9jcmwvTWljcm9zb2Z0JTIwRGV2ZWxvcG1lbnQl
# MjBSb290JTIwQ2VydGlmaWNhdGUlMjBBdXRob3JpdHklMjAyMDE0LmNybDCBhQYI
# KwYBBQUHAQEEeTB3MHUGCCsGAQUFBzAChmlodHRwOi8vd3d3Lm1pY3Jvc29mdC5j
# b20vcGtpb3BzL2NlcnRzL01pY3Jvc29mdCUyMERldmVsb3BtZW50JTIwUm9vdCUy
# MENlcnRpZmljYXRlJTIwQXV0aG9yaXR5JTIwMjAxNC5jcnQwDQYJKoZIhvcNAQEL
# BQADggIBAE7qHQz+04LRdV/wMN93jayDVrA0zb6nB/N0rO48Sl0f2r0q9mj7KjdD
# lwl5LAG9odSadTiUl2LI63O8iZRDyyi0S1W58VbVJOLLp/BGQNb+MP4pMlPtOSED
# czCVLWqxhwPfUXzxRhP59QVE3Q+SYzpxzM+gVfFtLZUh4HZUZJXZZo9/ayXFioYZ
# cTTFUyL23FWb2rrmpfL39ix2RtJh3kja43AeVeAxjFv0s8Id+v9+sY213v6U/QFK
# y/ofTA20WnIigrYrI1MKnS7fVT38GLMyW06xrwC/mviRVi5xTkRSxzV503VKuESl
# pflnRM8nMNx4T34J1aFRwGs/iuNZC3HvypI1ndEqt4x1IEOSdC4S3056e1mmDGWH
# yptpU3UirNn22Q85vsaTSeFt2GNy5YZcKcfV26X5XpM5sMWEyXHiFVSmzIqpHkYk
# TeQ8n8EF34w0M24FwaFA2agLlUlzDANPzMN8UiPepDbFzYI9y/jwEnzYHC1dt7oF
# fsAOGZ9Se1M/Bpq9lCdiT1UDluKJr/PMZ5UwLRq0ALtVx2+LkH1NTyxCUIRjPit2
# wwT3VuDgMXZQ5QIMAKoOT/SkKR8SlmKoZNzxww4m12Ljxq2kaABnMEEBFx/fYWSL
# gx03uoLxaJ20aZ+tgtzbE1LWTFULDM3z1blZxvdxi33/cV492d4AMYICyDCCAsQC
# AQEwgZQwfTELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
# BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEnMCUG
# A1UEAxMeTWljcm9zb2Z0IERldmVsb3BtZW50IFBDQSAyMDE0AhMzAAAF4EuwoimM
# 649KAAAAAAXgMA0GCWCGSAFlAwQCAQUAoIIBBDAZBgkqhkiG9w0BCQMxDAYKKwYB
# BAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0B
# CQQxIgQg9dqdhF4OR7gDjF8vJdrLT9KZqbWkhiPfaXbvLkc5VLgwPAYKKwYBBAGC
# NwoDHDEuDCxzUFk3eFBCN2hUNWc1SEhyWXQ4ckRMU005VnVaUnVXWmFlZjJlMjJS
# czU0PTBaBgorBgEEAYI3AgEMMUwwSqAkgCIATQBpAGMAcgBvAHMAbwBmAHQAIABX
# AGkAbgBkAG8AdwBzoSKAIGh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS93aW5kb3dz
# MA0GCSqGSIb3DQEBAQUABIIBAEI4q0TO1h8XrZkhb/cEXSf3trvRtgdR6QGCTNXq
# NYYx2vd+tW6GCAiG64jL/Z47uBULsCwy+UibRWLAm+3Z4j50kPwWS8fujbEV/DW4
# HSpdfyk0X0KA1mDRSsJR8CXgx5oLfzUZGIXJSVqD4KiiAjqgTgLWy3/M8f8PQy6V
# eeiSI+rIpyHTqSnlHEO0YlildKNH2oenCQetMiLZ+OC1vByeiGS+ywvDX3u4r9Q0
# 57PZS7bm5pFBwmSPCKxhlpyHlioSu3I0VX/zvTXx0UzAGz2Zf1Y0prMriyNB4b4v
# ADHOQ26nhxH1p2dYQ77W83amDWz9zuUeBbS2Qfw2JCAlkdE=
# SIG # End signature block
